// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classEventClock
#define classEventClock

class EventClock;

#include <sys/time.h>
#include <string>
#include <iostream>

//! Event clock.
/*!
	An object of this class is used to model the time reference for the application.

	The time reference is updated by the MainLoop when new events (EventInterface timeout or active IOInterface filehandles) need to be dispatched. That means that all events dispatched in that round perceive the same time (returned by the GetTime() method).
*/
class EventClock {
friend class EventInterface;

private:
	//! Current clock in milliseconds.
	unsigned long long mTime;

public:
	//! The datatype for time.
	typedef unsigned long long tTime;

	//! Constructor.
	EventClock();

	//! Sets the time to now.
	void Update();
	//! Returns the clock in milliseconds.
	unsigned long long GetTime() {return mTime;}

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;
};

#endif
