// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "EventInterface.h"
#include "Constants.h"
#define THISCLASS EventInterface

THISCLASS::EventInterface(EventInterfaceList *evlist):
		mEventActive(false), mEventTime(0),
		mEventInterfaceList(0), mEventInterfaceNext(0) {

	if (evlist) {
		evlist->AddEventInterface(this);
	}
}

THISCLASS::~EventInterface() {
	if (mEventInterfaceList) {mEventInterfaceList->RemoveEventInterface(this);}
}

void THISCLASS::SetNextEvent(int msec) {
	if (! mEventInterfaceList) {return;}
	if (! mEventInterfaceList->mEventClock) {return;}

	EventClock::tTime evtime=mEventInterfaceList->mEventClock->GetTime();
	evtime+=msec*1000;
	if ((! mEventActive) || (evtime<mEventTime)) {
		mEventTime=evtime;
		mEventActive=true;
	}
}

void THISCLASS::SetNextEventAbsolute(EventClock::tTime evtime) {
	if ((! mEventActive) || (evtime<mEventTime)) {
		mEventTime=evtime;
		mEventActive=true;
	}
}

void THISCLASS::DispatchEvent() {
	if (! mEventActive) {return;}

	// Execute the event if it is within this millisecond.
	// Much better precision can't be achieved (and is not necessary), since the select() function returns at +-200us of the timeout and has roughly 1000us minimum waiting time.
	if (mEventTime > mEventInterfaceList->mEventClock->mTime+1000) {return;}

	mEventActive=false;
	this->OnEvent();
}
