// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "EventInterfaceList.h"
#include <cmath>
#define THISCLASS EventInterfaceList

#include <iostream>

THISCLASS::EventInterfaceList():
		mEventInterfaceFirst(0), mEventClock(0) {

	mEventClock=new EventClock();
}

THISCLASS::~EventInterfaceList() {
	delete mEventClock;
}

EventInterface *THISCLASS::GetEventInterfaceNext(EventInterface *cur) {
	return (cur ? cur->mEventInterfaceNext : 0);
}

bool THISCLASS::AddEventInterface(EventInterface *sif) {
	if (! sif) {return true;}
	if (sif->mEventInterfaceList) {return false;}
	sif->mEventInterfaceNext=mEventInterfaceFirst;
	sif->mEventInterfaceList=this;
	mEventInterfaceFirst=sif;
	return true;
}

bool THISCLASS::RemoveEventInterface(EventInterface *sif) {
	if (! sif) {return true;}
	if (sif->mEventInterfaceList!=this) {return false;}

	// If it is the first element in the list
	if (mEventInterfaceFirst==sif) {
		mEventInterfaceFirst=sif->mEventInterfaceNext;
			sif->mEventInterfaceList=0;
			sif->mEventInterfaceNext=0;
			return true;
	}

	// If it is any other element in the list
	EventInterface *cur=mEventInterfaceFirst;
	while (cur) {
		if (cur->mEventInterfaceNext==sif) {
			cur->mEventInterfaceNext=sif->mEventInterfaceNext;
			sif->mEventInterfaceList=0;
			sif->mEventInterfaceNext=0;
			return true;
		}
		cur=cur->mEventInterfaceNext;
	}

	return false;
}

EventClock::tTime THISCLASS::GetNextEventTimeout() {
	EventClock::tTime nexteventtime=mEventClock->GetTime()+10000000;
	EventInterface *curevent=mEventInterfaceFirst;
	while (curevent) {
		if (curevent->HasNextEvent()) {
			EventClock::tTime cureventtime=curevent->GetNextEventTime();
			if (cureventtime<nexteventtime) {nexteventtime=cureventtime;}
		}
		curevent=curevent->GetEventInterfaceNext();
	}

	mEventClock->Update();
	return nexteventtime - mEventClock->GetTime();
}

void THISCLASS::Report(const std::string &name, std::ostream &out, int ccindent) const {
	std::string indent(ccindent*2, ' ');
	std::cout << indent << name << " = EventInterfaceList " << this << std::endl;
	mEventClock->Report("mEventClock", out, ccindent+1);
	
	int cc=0;
	EventInterface *cur=mEventInterfaceFirst;
	while (cur) {
		cc++;
		if ((cur->mEventActive) && (mEventClock)) {
			EventClock::tTime diff=cur->mEventTime - mEventClock->GetTime();
			int sec=(int)floor(diff/1000000);
			int usec=diff-sec*1000000;
			std::cout << indent << "  " << cc << " = EventInterface " << cur << ": active in " << sec << " sec " << usec << " usec" << std::endl;
		} else {
			std::cout << indent << "  " << cc << " = EventInterface " << cur << ": (inactive)" << std::endl;
		}
		cur=cur->mEventInterfaceNext;
	}
}
