// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classEventInterfaceList
#define classEventInterfaceList

class EventInterfaceList;

#include "EventInterface.h"

//! Implements a linked list for EventInterface objets.
class EventInterfaceList {
friend class EventInterface;

protected:
	//! The first EventInterface object.
	EventInterface *mEventInterfaceFirst;
	//! The event clock. It defines how the program experiences the flow of time.
	EventClock *mEventClock;

public:
	//! Constructor.
	EventInterfaceList();
	//! Destructor.
	~EventInterfaceList();

	//! Returns the EventClock.
	EventClock* GetEventClock() const {return mEventClock;}
	//! Returns the first EventInterface object.
	EventInterface *GetEventInterfaceFirst() {return mEventInterfaceFirst;}
	//! Returns the next EventInterface object.
	EventInterface *GetEventInterfaceNext(EventInterface *cur);

	//! Returns the timeout until the next event.
	EventClock::tTime GetNextEventTimeout();
	//! Updates the event clock.
	void UpdateClock() {mEventClock->Update();}

	//! Adds an EventInterface to the list.
	bool AddEventInterface(EventInterface *sif);
	//! Removes an EventInterface from the list.
	bool RemoveEventInterface(EventInterface *sif);

	//! Writes a report of this object to the given stream. This is mainly useful for debugging.
	void Report(const std::string &name, std::ostream &out, int ccindent=0) const;
};

#endif
