// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "IOInterface.h"
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include <errno.h>
#define THISCLASS IOInterface

THISCLASS::IOInterface(IOInterfaceList *iolist):
		mFileHandle(-1), mLogInput(), mLogOutput(),
		mIOInterfaceList(0), mIOInterfaceNext(0) {

	if (iolist) {
		iolist->AddIOInterface(this);
	}
}

THISCLASS::~IOInterface() {
	if (mIOInterfaceList) {mIOInterfaceList->RemoveIOInterface(this);}
}

void THISCLASS::OnInputAvailable() {
	if (mFileHandle<0) {return;}

	unsigned char buffer[255];
	int len = read(mFileHandle, buffer, 255);
	if (len==0) {
		close(mFileHandle);
		mFileHandle=-1;
		OnClose();
		return;
	}

	if (mLogInput.is_open()) {mLogInput.write((char *)buffer, len);}
	OnDataArrival(buffer, len);
}

bool THISCLASS::Send(unsigned char *data, int len) {
	if (mLogOutput.is_open()) {mLogOutput.write((char *)data, len);}
	if (mFileHandle<0) {return false;}
	//std::cout << "Sending " << len << " chars <" << data << ">" << std::endl;
	int written=write(mFileHandle, data, len);
	//std::cout << "written " << written << std::endl;
	return (written==len);
}

bool THISCLASS::Send(const std::string &str) {
	if (mLogOutput.is_open()) {mLogOutput << str;}
	if (mFileHandle<0) {return false;}
	int len=str.length();
	//std::cout << "Sending " << len << " chars <" << str << ">" << std::endl;
	int written=write(mFileHandle, str.c_str(), len);
	//std::cout << "written " << written << std::endl;
	return (written==len);
}

bool THISCLASS::SetInputLogFile(const std::string &file, bool append) {
	if (! mLogInput.is_open()) {mLogInput.close();}
	if (file=="") {return true;}
	
	if (append) {
		mLogInput.open(file.c_str(), std::ios::out | std::ios::app);
	} else {
		mLogInput.open(file.c_str(), std::ios::out | std::ios::trunc);
	}

	if (mLogInput.is_open()) {
		return true;
	}

	return false;
}

bool THISCLASS::SetOutputLogFile(const std::string &file, bool append) {
	if (! mLogOutput.is_open()) {mLogOutput.close();}
	if (file=="") {return true;}

	if (append) {
		mLogOutput.open(file.c_str(), std::ios::out | std::ios::app);
	} else {
		mLogOutput.open(file.c_str(), std::ios::out | std::ios::trunc);
	}

	if (mLogOutput.is_open()) {
		return true;
	}

	return false;
}

