// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "IOInterfaceSerial.h"
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include "termioscms.h"
#define THISCLASS IOInterfaceSerial

THISCLASS::IOInterfaceSerial(IOInterfaceList *iolist, const std::string &device, int baudrate):
		IOInterface(iolist), mDevice(device), mBaudRate(baudrate), mTermiosLocalMode(0) {

}

void THISCLASS::SetDevice(const std::string &device) {
	mDevice=device;
}

void THISCLASS::SetBaudRate(int baudrate) {
	mBaudRate=baudrate;
}

void THISCLASS::SetTermiosLocalMode(int mode) {
	mTermiosLocalMode=mode;
}

bool THISCLASS::Reopen() {
	if (! Close()) {return false;}
	return Open();
}

bool THISCLASS::Close() {
	if (mFileHandle<0) {return true;}
	OnBeforeClose();
	std::cout << "attempt to close serial device " << mDevice << " (fh " << mFileHandle << ")" << std::endl;
	close(mFileHandle);
	std::cout << " -- serial device " << mDevice << " (fh " << mFileHandle << ") closed\n" << std::endl;
	mFileHandle=-1;
	return true;
}

bool THISCLASS::Open() {
	if (mFileHandle>=0) {return true;}

	// Don't touch anything if the device properties are not available
	if ((mDevice.length()==0) || (mBaudRate==0)) {
		mFileHandle=-1;
		return false;
	}

	// Open serial port
	std::cout << "attempt to open serial device " << mDevice << std::endl;
	mFileHandle=open(mDevice.c_str(), O_RDWR | O_NOCTTY);
	if (mFileHandle<0) {
		std::cout << " -- serial device " << mDevice << ": error " << mFileHandle << std::endl;
		mFileHandle=-1;
		OnAfterOpenError();
		return false;
	}
	std::cout << " -- serial device " << mDevice << " opened (fh " << mFileHandle << ")" << std::endl;

	// Set port Settings
	struct termios newtio;
	bzero(&newtio, sizeof(newtio));
	newtio.c_cflag = BaudRateConstant(mBaudRate) | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR | IGNBRK;
	newtio.c_oflag = 0;
	newtio.c_lflag = mTermiosLocalMode;
	newtio.c_cc[VTIME] = 5;
	newtio.c_cc[VMIN] = 1;

	tcflush(mFileHandle, TCIOFLUSH);
	std::cout << " -- serial device " << mDevice << " tcflush done (fh " << mFileHandle << ")" << std::endl;
	tcsetattr(mFileHandle, TCSANOW, &newtio);
	std::cout << " -- serial device " << mDevice << " tcsetattr done (fh " << mFileHandle << ")" << std::endl;

	//res=SioDTR(mPort, WSC_SET_LINE);
	//res=SioRTS(mPort, WSC_SET_LINE);
	
	// Set non-blocking mode
	fcntl(mFileHandle, F_SETFL, O_NONBLOCK);
	std::cout << " -- serial device " << mDevice << " fcntl done (fh " << mFileHandle << ")" << std::endl;

	OnAfterOpen();
	return true;
}

unsigned long THISCLASS::BaudRateConstant(int baudrate) {
	if (baudrate==50) {return B50;}
	if (baudrate==75) {return B75;}
	if (baudrate==110) {return B110;}
	if (baudrate==134) {return B134;}
	if (baudrate==150) {return B150;}
	if (baudrate==200) {return B200;}
	if (baudrate==300) {return B300;}
	if (baudrate==600) {return B600;}
	if (baudrate==1200) {return B1200;}
	if (baudrate==1800) {return B1800;}
	if (baudrate==2400) {return B2400;}
	if (baudrate==4800) {return B4800;}
	if (baudrate==9600) {return B9600;}
	if (baudrate==19200) {return B19200;}
	if (baudrate==38400) {return B38400;}
	if (baudrate==57600) {return B57600;}
	if (baudrate==115200) {return B115200;}
	if (baudrate==230400) {return B230400;}
	//if (baudrate==460800) {return B460800;}
	return B1200;
}
