// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classIOInterfaceSerial
#define classIOInterfaceSerial

#include <string>
#include "IOInterface.h"

//! IOInterface for a serial connection.
/*!
	This implements an IOInterface for a serial port (usually /dev/ttyS*).
*/
class IOInterfaceSerial: public IOInterface {

private:
	//! The device.
	std::string mDevice;
	//! The baud rate.
	int mBaudRate;
	//! Whether ICANON is set or not.
	int mTermiosLocalMode;

protected:
	//! Converts a baud rate into a baud rate constant.
	unsigned long BaudRateConstant(int baudrate);

public:
	//! Constructor.
	IOInterfaceSerial(IOInterfaceList *iolist, const std::string &device="", int baudrate=9600);

	//! Called before a reset.
	virtual void OnBeforeClose() {}
	//! Called after a reset.
	virtual void OnAfterOpen() {}
	//! Called after a reset.
	virtual void OnAfterOpenError() {}

    //! Returns the device.
    std::string GetDevice() const {return mDevice;}
    //! Returns the baud rate.
    int GetBaudRate() const {return mBaudRate;}
    //! Returns the termios local mode flags.
    bool GetTermiosLocalMode() const {return mTermiosLocalMode;}

    //! Sets the device. The new device is used after the next Reopen() or Open() call.
    void SetDevice(const std::string &device);
    //! Sets the baud rate. The new baud rate is used after the next Reopen() or Open() call.
    void SetBaudRate(int baudrate);
    //! Sets the ICANON flag. The new setting is used after the next Reopen() or Open() call.
    void SetTermiosLocalMode(int mode);

	//! Opens the serial port if it is closed. Returns true if the serial port was successfully opened or if it was open already. Note that calling Open() on an open device won't do anything. Call Reopen() if you want to close and open it again.
	bool Open();
	//! Closes the serial port if it is open. Returns true if the serial port was successfully closed or if it was closed already.
	bool Close();
	//! Closes (if necessary) and opens the serial port. Returns true if the serial port has been opened.
	bool Reopen();
};

#endif
