// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef fileMipDefinitions
#define fileMipDefinitions

//! Mip Definitions.
/*!
	This file contains the definitions of various structs used by the MIP devices. For detailed information, please refer to the MIP manual.
*/

//! SysParam struct
typedef struct T_SysParam {
/*nr offSet */

/* system informations */
/* 1 0 */	char AxisNumber;				//!< No. of the axis = MIP-Bus Address
/* 6 1 */	char GroupAddress;				//!< Group address for multicast messages
/* 2 2 */	int StartupMode;				//!< No. of active mode after startup
/* 3 4 */	long RS232BaudRate;				//!< Baudrate RS232
/* 4 8 */	int RS232Mode;					//!< 0 = N81
/* 5 10 */	long RS485BaudRate;				//!< Baudrate RS485
/* 7 14 */	TSysCfg SysConfig;				//!< System configuration. 16 bits: see definition below
/* 8 16 */	TsysCfg2 SysCongig2;			//!< System configuration 2. 16 bits: see definition below
/* 9 18 */	int SupplyVolt;					//!< Supply Voltage (needed for MIP-EC current regulation)
/* 0 20 */	int NbOfChanges;				//!< no. of parameter changes

/* motor/encoder/gear informations */
/*10 22 */	int EncoderResolution;			//!< encoder resolution [impulse/round]
/*11 24 */	int MaxVelocity;				//!< in [rpm]
/*12 26 */	word MinAccelTime;				//!< time [ms] to accelerate 0 to MaxVelocity
/*13 28 */	int PeakCurrent;				//!< max. peak current in [mA]
/*14 30 */	int ContCurrent;				//!< max. continuous current in [mA]
/*15 32 */	float GearRatio;				//!< total transmission ratio
/*16 36 */	float Inertia;					//!< total inertia (rotor + load) in [g*cm2]
/*17 40 */	float TorqueConst;				//!< in [Nm/A]
/*18 44 */	float SpeedConst;				//!< in [rpm/V] (needed for MIP-EC current regulation)
/*19 48 */	int CurDelta;					//!< third parameter for EC-current regulation

/* user informations */
/*20 50 */	long PosErrorMax;				//!< max. position error in [qc]
/*21 54 */	long UpPositionLimit;			//!< upper position limit [qc]
/*22 58 */	int EnableUpLimit;				//!< 1 = enable, 0 = disable
/*23 60 */	long LoPositionLimit;			//!< lower position limit [qc]
/*24 64 */	int EnableLoLimit;				//!< 1 = enable, 0 = disable
/*25 66 */	int PosGainP;					//!< P gain for position regulation
/*26 68 */	int PosGainI;					//!< I gain for position regulation
/*27 70 */	int PosGainD;					//!< D gain for position regulation
/*28 72 */	int CurGainP;					//!< P gain for current regulation
/*29 74 */	int CurGainI;					//!< I gain for current regulation
/*30 76 */	int HomeType;					//!< see possible types below
/*31 78 */	float HomeVelocity;				//!< velocity of home seek [qc/ms]
/*32 82 */	int CurrentThreshold;			//!< current index threshold [mA]
/*33 84 */	long HomeOffSet;				//!< relative offSet to home [qc]
/*34 88 */	int PLMaxVelocity;				//!< upper limit for pulse length velocity measurement
/*35 90 */	int AuxEncoderMode;				//!< 0 = without ext. encoder, 1 = with 'a-dir' encoder, 2 = with 'a-b' encoder
/*36 92 */	float AuxEncoderGain;			//!< Auxiliary encoder gain
/*37 96 */	int BrakeGain;					//!< low level braking gain
/*38 98 */	int HomePositionHi;				//!< position value Set after Homing (Hi-Word)
/*39 100 */	int HomePositionLo;				//!< same (Lo-Word)

/* utility information, autocalculated, read only */
/*40 102 */	long OneRevolution;				//!< one full rotor revolution [qc]
/*41 106 */	long OneLoadRevolution;			//!< one full load revolution [qc].

/* the following parameters belong to the extended parameter structure */
/*42 110 */	word ExtendTag;					//!< if == EXTEND_TAG, following parameters exist
/*43 112 */	long ExtParam[35];				//!< place for 35 extended parameters (43..77)
/*78 252 */	long SysParamCRC;				//!< CRC of system parameters
} T_SysParam;

#define EXTEND_TAG 54321 /* this value indicates the presence of an extended parameter Set */

//! Bit definition for the SysConfig parameter
enum eMipSysConfig {
/* 0  */	StartCurReg = 0x1;		//!< start current regulator after startup
/* 1  */	StartPosReg = 0x2;		//!< start position regulator after startup
/* 2  */	StopCurReg = 0x4;		//!< stop the current regulator after a STOP or HALT
/* 3  */	BrakeAtStop = 0x8;		//!< brake after a STOP or HALT
/* 4  */	CheckEnable = 0x10;		//!< check the enable input
/* 5  */	CheckLeft = 0x20;		//!< check left end switch input
/* 6  */	CheckRight = 0x40;		//!< check right end switch input
/* 7  */	CheckDigOvld = 0x80;	//!< check digital overload input
/* 8  */	ClearStop = 0x100;		//!< enable clearing stop error
/* 9  */	ClearLeft = 0x200;		//!< enable clearing left error while left active
/*10  */	ClearRight = 0x400;		//!< enable clearing right error while right active
/*11  */	StopCurReg2 = 0x800;	//!< stop the cur. regul. after a pos. regulation stop
/*12  */	CycloidalProf = 0x1000;	//!< use cycloidal profile instead of parabolic
/*13  */	CheckEncoder = 0x2000;	//!< check encoder signals: needs special hardware
/*14  */	b14 = 0x4000;			//!< unused
/*15  */	ConfigDC = 0x8000;		//!< configure MIP-EC 50/5 for DC motor operation
};

//! Bit definition for the SysConfig2 parameter
enum eMipSysConfig2 {
/* 0  */	EnableWdog = 1;		//!< enable watchdog feature
/* 1  */	ClearWdog = 2;		//!< enable clearing the watchdog error
/* 2  */	IgnoreWdog = 4;		//!< ignore watchdog error at startup
/* 3  */	DisOvrcur = 8;		//!< disable overcurrent protection
};

#endif
