// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MipManager.h"
#include <sstream>
#define	THISCLASS MipManager

THISCLASS::MipManager(IOInterfaceList *iolist, EventInterfaceList *evlist):
		mIOInterfaceList(iolist), mEventInterfaceList(evlist) {

	if (mIOInterfaceList==0) {mIOInterfaceList=new IOInterfaceList();}
	if (mEventInterfaceList==0) {mEventInterfaceList=new EventInterfaceList();}

	for (int i=0; i<mMipBusMax; i++) {
		mMipBus[i]=0;
	}
}

THISCLASS::~MipManager() {
	for (int i=0; i<mMipBusMax; i++) {
		if (mMipBus[i]) {delete mMipBus[i];}
	}
}

void THISCLASS::SetLogInterface(MipLogInterface *miplog) {
	if (mMipLogInterface) {mMipLogInterface->SetMipManager(0);}
	mMipLogInterface=miplog;
	if (mMipLogInterface) {mMipLogInterface->SetMipManager(this);}
}

MipBus *THISCLASS::FindMipBus(const std::string &device) {
	int freeid=mMipBusMax;
	for (int i=0; i<mMipBusMax; i++) {
		if (mMipBus[i]) {
			if (mMipBus[i]->GetDevice()==device) {
				return mMipBus[i];
			}
		} else {
			freeid=i;
		}
	}

	if (freeid==mMipBusMax) {return 0;}
	mMipBus[freeid]=new MipBus(mIOInterfaceList, mEventInterfaceList, device);
	mMipBus[freeid]->SetLogInterface(mMipLogInterface);
	return mMipBus[freeid];
}

void THISCLASS::Report(const std::string &name, std::ostream &out, int ccindent) const {
	std::string indent(ccindent*2, ' ');
	std::cout << indent << name << " = MipManager " << this << std::endl;
	std::cout << indent << "  mMipLogInterface = MipLogInterface " << mMipLogInterface << std::endl;
	mIOInterfaceList->Report("mIOInterfaceList", out, ccindent+1);
	mEventInterfaceList->Report("mEventInterfaceList", out, ccindent+1);
	for (int i=0; i<mMipBusMax; i++) {
		if (mMipBus[i]) {
			std::ostringstream name;
			name << "mMipBus[" << i << "]";
			mMipBus[i]->Report(name.str(), out, ccindent+1);
		}
	}
}
