// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipMessage
#define classMipMessage

//! MipBus Protocol Message.
/*!
	This (abstract) class models a MIP bus message. MIP bus messages are implemented in the classes MipRxMessage (received messages) and MipTxMessage (messages to transmit).

	For information about the MIP bus message format, please refer to the MIP manual.
*/
class MipMessage {

public:
	//! Defines the enum eType.
	#include "MipMessageTypes.inc.h"

protected:
	//! The device ID of the destination (the device that receives the message).
	int mDestination;
	//! The device ID of the source (the device that sends the message).
	int mSource;
	//! The message type.
	eType mType;
	//! The data.
	unsigned char mData[1024];

public:
	//! Constructor.
	MipMessage(int src=0, int dest=0, eType type=sNone);

	//! Returns the destination.
    int GetDestination() const {return mDestination;}
    //! Returns the source.
    int GetSource() const {return mSource;}
    //! Returns the type.
    eType GetType() const {return mType;}
	//! Returns the length.
	virtual int GetLength() const = 0;

    //! Sets the destination.
    void SetDestination(int dest);
    //! Sets the broadcast destination.
    void SetBroadcast() {SetDestination(255);}
    //! Sets the source.
    void SetSource(int src);
	//! Sets the message type.
	void SetType(eType type);

	//! Calculates the checksum of the packet.
	unsigned int CalcCRC();
};

#endif
