// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "MipRxMessage.h"
#define	THISCLASS MipRxMessage
#include <stdio.h>

THISCLASS::MipRxMessage():
	MipMessage(), mReadIndex(5), mReceiveIndex(0), mErronous(false), mComplete(false) {
}

void THISCLASS::AddReceivedData(unsigned char *data, int len) {
	if (mErronous) {return;}
	if (mComplete) {return;}
	bool headerread=(mReceiveIndex>5);

	if (len>1023-mReceiveIndex) {len=1023-mReceiveIndex;}
	for (int i=0; i<len; i++) {
		mData[mReceiveIndex++]=data[i];
	}

	if (mReceiveIndex>5) {
		if (! headerread) {
			if (! ReadHeader()) {
				mErronous=true;
				mComplete=true;
				return;
			}
		}

		if (mReceiveIndex>=mDataLength+7) {
			mErronous=(! CheckCRC());
			mComplete=true;
		}
	}
}

bool THISCLASS::ReadHeader() {
	mDestination=(int)(mData[0]);
	if (! ReadLength()) {return false;}
	mSource=(int)(mData[3]);
	mType=(eType)(mData[4]);
	return true;
}

bool THISCLASS::ReadLength() {
	int len=mData[1]*0x100+mData[2];
	int checksum = (len)&0x1F+((len/32)&0x1F)+(len)/1024;
	if (checksum & 0x3F) return false;

	mDataLength=len & 1023;
	return true;
}

unsigned char THISCLASS::ReadByte() {
	return mData[mReadIndex++];
}

unsigned short int THISCLASS::ReadUnsignedInteger16() {
	unsigned char *buf;
	unsigned short int value;
	buf=(unsigned char*)&value;
	buf[1]=mData[mReadIndex++];
	buf[0]=mData[mReadIndex++];
	return value;
}

short int THISCLASS::ReadSignedInteger16() {
	unsigned char *buf;
	short int value;
	buf=(unsigned char*)&value;
	buf[1]=mData[mReadIndex++];
	buf[0]=mData[mReadIndex++];
	return value;
}

unsigned int THISCLASS::ReadUnsignedInteger32() {
	unsigned char *buf;
	unsigned int value;
	buf=(unsigned char*)&value;
	buf[3]=mData[mReadIndex++];
	buf[2]=mData[mReadIndex++];
	buf[1]=mData[mReadIndex++];
	buf[0]=mData[mReadIndex++];
	return value;
}

int THISCLASS::ReadSignedInteger32() {
	unsigned char *buf;
	int value;
	buf=(unsigned char*)&value;
	buf[3]=mData[mReadIndex++];
	buf[2]=mData[mReadIndex++];
	buf[1]=mData[mReadIndex++];
	buf[0]=mData[mReadIndex++];
	return value;
}

float THISCLASS::ReadFloat() {
	unsigned char *buf;
	float value;
	buf=(unsigned char*)&value;
	buf[3]=mData[mReadIndex++];
	buf[2]=mData[mReadIndex++];
	buf[1]=mData[mReadIndex++];
	buf[0]=mData[mReadIndex++];
	return value;
}

bool THISCLASS::ReadBool() {
	return (mData[mReadIndex++] != 0);
}

bool THISCLASS::CheckCRC() {
	unsigned int crc=CalcCRC();
	unsigned char *buf;
	buf=(unsigned char*)&crc;
	int crcindex=mDataLength+5;
	return (mData[crcindex]==buf[1]) && (mData[crcindex+1]==buf[0]);
}
