// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMipTxMessage
#define classMipTxMessage

#include "MipMessage.h"

//! MipBus Protocol Transmit Message.
/*!
	This class prepares a message for transmission to a MIP. It writes the header with the correct length and header checksum and calculates the CRC of the message. Furthermore, it contains methods to pack the arguments (integers, floats, ...) into the message.
*/
class MipTxMessage: public MipMessage {

private:
	int mWriteIndex;			// index of the next data byte to write

public:
	//! Constructor.
	MipTxMessage(int src=0, int dest=0, eType type=sNone);

	//! Returns the length.
	int GetLength() const {return mWriteIndex+2;}
	//! Returns the complete packet.
	unsigned char *GetPacket();

	//! Writes a command (byte) to the data.
	void WriteCommand(unsigned char value) {WriteByte(value);}
	//! Writes a byte to the data.
	void WriteByte(unsigned char value);
	//! Writes an unsigned 16bit integer to the data.
	void WriteUnsignedInteger16(unsigned short int value);
	//! Writes a signed 16bit integer to the data.
	void WriteSignedInteger16(short int value);
	//! Writes an unsigned 32bit integer to the data.
	void WriteUnsignedInteger32(unsigned int value);
	//! Writes a signed 32bit integer to the data.
	void WriteSignedInteger32(int value);
	//! Writes a float to the data.
	void WriteFloat(float value);
	//! Writes a bool to the data.
	void WriteBool(bool value);

	//! Writes the message header.
	void WriteHeader();
	//! Writes the length (including the header checksum) to the message.
	void WriteLength();
	//! Calculates the CRC and adds it to the message.
	void WriteCRC();
};

#endif
