// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#ifndef classMyMip
#define classMyMip

class MyMip;

#include <string>
#include <mip/MipDevice.h>
#include <mip/Constants.h>

/*!
	Sample Mip class.

	This class implements the desired functionality. It sends commands to the MIP device and processes the received replies. Furthermore, it periodically checks the motor control status.
*/
class MyMip: public EventInterface, public MipDevice {

private:
	//! Whether to drive the motors.
	bool mDriveMotor;
	//! Where we are.
	double mAngleNow;
	//! Whether the motor is in home position.
	bool mHome;
	//! Whether the motor is moving.
	bool mMoving;
	//! Whether the zero-position has been found.
	bool mReferenced;
	//! The error code.
	int mError;
	//! The warning code.
	int mWarning;

	//! Sends an angle to the Mip.
	void SendAngle(double angle);

public:
	//! Constructor.
	MyMip(MipManager *mm, const std::string &device, int baudrate, int deviceid);

	//! Initialize.
	void Initialize();
	//! Terminate.
	void Terminate();

	// Overwritten MipDevice methods
	virtual void OnReceivedReadAxisStatus(int cmdid, eAxisStatusFlags status);
	virtual void OnReceivedReadError(int cmdid, eError error);
	virtual void OnReceivedClearError(int cmdid) {}
	virtual void OnReceivedReadWarning(int cmdid, eWarning warning);
	virtual void OnReceivedClearWarning(int cmdid) {}
	virtual void OnReceivedReadVersion(int cmdid, int command_interpreter_version, int runtime_library_version, int hardware_revision, int bootrom_version, int application_number, int application_version) {}
	virtual void OnReceivedSetPosVelocity(int cmdid) {}
	virtual void OnReceivedSetWaitMode(int cmdid) {}
	virtual void OnReceivedMoveAbsolute(int cmdid) {}
	virtual void OnReceivedMoveRelative(int cmdid) {}
	virtual void OnReceivedStopMotion(int cmdid) {}
	virtual void OnReceivedFindHomeSys(int cmdid) {}
	virtual void OnReceivedReadPosition(int cmdid, int position);
	virtual void OnReceivedReadVelocity(int cmdid, int position) {}
	virtual void OnReceivedReadVelocityMean(int cmdid, int position) {}
	virtual void OnReceivedReadCurrent(int cmdid, int position) {}
	virtual void OnReceivedCheckCRC(int cmdid, bool ok, int crc) {}
	
	// Overwritten EventInterface methods
	void OnEvent();

	//! Converts positions to angles.
	int AngleToPosition(double angle);
	//! Converts angles to positions.
	double PositionToAngle(int position);
	//! Returns the angle associated with the shortest motor movement.
	double ShortestAngle(double anglemust, double anglenow);

	//! Converts degrees to radians.
	double DegToRad(double angle) {return angle/180*PI;}
	//! Converts radians to degrees.
	double RadToDeg(double angle) {return angle/PI*180;}
};

#endif
