// Copyright (c) 2004, Thomas Lochmatter, thl@lothosoft.ch

#include "SampleApplication.h"
#include <string>
#include <iostream>
#define THISCLASS SampleApplication

int main(int argc, char *argv[]) {
	SampleApplication app(argc, argv);
	return app.Run();
}

THISCLASS::SampleApplication(int argc, char **argv) {
	// Create a Mip Manager
	// All mip objects are attached to the mip manager. The manager transparently handles a set of
	// serial interfaces which several attached mip devices.
	mMipManager=new MipManager(mIOInterfaceList, mEventInterfaceList);

	// Create a Mip object to access the mip device number 1 on ttyS0 (default)
	std::string device="/dev/ttyS0";
	int speed=4800;
	int deviceid=1;
	if (argc>1) {device=argv[1];}
	std::cout << "Selected MIP: DeviceID " << 0 << " on serial port " << device << " (" << speed << " baud)" << std::endl;
	mMyMip=new MyMip(mMipManager, device, speed, deviceid);

	// You may create more mip devices on the same or other serial interfaces
	//mMyMip2=new MyMip(mipmanager, "/dev/ttyS0", 4800, 2);
	//mMyMip3=new MyMip(mipmanager, "/dev/ttyS1", 4800, 1);
	//mMyMip4=new MyMip(mipmanager, "/dev/ttyS1", 4800, 2);
}

THISCLASS::~SampleApplication() {
	delete mMipManager;
}

void THISCLASS::OnInitialize() {
	// Initialize the MyMip object
	std::cout << "Initializing ..." << std::endl;
	mMyMip->Initialize();
	std::cout << "Ready" << std::endl;

	// Dump the structure to STDOUT (for debugging)
	std::cout << std::endl;
	std::cout << "---------- BEGIN STRUCTURE DUMP ----------" << std::endl;
	Report("SampleApplication::MainLoop", std::cout);
	std::cout << "----------" << std::endl;
	mMipManager->Report("mMipManager", std::cout);
	std::cout << "---------- END STRUCTURE DUMP ----------" << std::endl;
	std::cout << std::endl;
}

void THISCLASS::OnTerminate() {
	// Cleanup the MyMip object
	std::cout << "Quitting ..." << std::endl;
	mMyMip->Terminate();
}
